@@ -22,6 +22,8 @@ from skrobot.coordinates.math import rotation_angle
 from skrobot.coordinates.math import rotation_matrix
 from skrobot.coordinates.math import rpy2quaternion
 from skrobot.coordinates.math import rpy_angle
+from skrobot.coordinates.math import wxyz2xyzw
+from skrobot.coordinates.math import xyzw2wxyz
 
 
 def transform_coords(c1, c2, out=None):
@@ -115,7 +117,7 @@ class Transform(object):
         Parameters
         ----------
         vec : numpy.ndarray(3,) or numpy.ndarray(3, n_points)
-            vector (or vectors) to be roated
+            vector (or vectors) to be rotated
 
         Returns
         -------
@@ -196,7 +198,8 @@ class Coordinates(object):
                  rot=None,
                  name=None,
                  hook=None,
-                 check_validity=True):
+                 check_validity=True,
+                 input_quaternion_order='wxyz'):
         if check_validity:
             if (isinstance(pos, list) or isinstance(pos, np.ndarray)):
                 T = np.array(pos, dtype=np.float64)
@@ -206,6 +209,16 @@ class Coordinates(object):
             if rot is None:
                 self._rotation = np.eye(3)
             else:
+                if input_quaternion_order == 'wxyz':
+                    pass
+                elif input_quaternion_order == 'xyzw':
+                    if np.array(rot).shape == (4,):
+                        rot = xyzw2wxyz(rot)
+                else:
+                    msg = "Invalid input_quaternion_order: "
+                    msg += "{}. Must be 'wxyz' or 'xyzw'.".format(
+                        input_quaternion_order)
+                    raise ValueError(msg)
                 self.rotation = rot
             if pos is None:
                 self._translation = np.array([0, 0, 0])
@@ -239,7 +252,7 @@ class Coordinates(object):
         Returns
         -------
         transform : skrobot.coordinates.base.Transform
-            corrensponding Transform to this coordinates
+            corresponding Transform to this coordinates
         """
         return Transform(self.worldpos(), self.worldrot())
 
@@ -275,7 +288,7 @@ class Coordinates(object):
     def rotation(self, rotation):
         """Set rotation of this coordinate
 
-        This setter checkes the given rotation and set it this coordinate.
+        This setter checks the given rotation and set it this coordinate.
 
         Parameters
         ----------
@@ -332,7 +345,7 @@ class Coordinates(object):
     def translation(self, translation):
         """Set translation of this coordinate
 
-        This setter checkes the given translation and set it this coordinate.
+        This setter checks the given translation and set it this coordinate.
 
         Parameters
         ----------
@@ -602,7 +615,7 @@ class Coordinates(object):
 
     @property
     def quaternion(self):
-        """Property of quaternion
+        """Property of quaternion in [w, x, y, z] format
 
         Returns
         -------
@@ -622,6 +635,39 @@ class Coordinates(object):
         """
         return matrix2quaternion(self._rotation)
 
+    @property
+    def quaternion_wxyz(self):
+        """Property of quaternion in [w, x, y, z] format
+
+        Returns
+        -------
+        q : numpy.ndarray
+            [w, x, y, z] quaternion
+        """
+        return matrix2quaternion(self._rotation)
+
+    @property
+    def quaternion_xyzw(self):
+        """Property of quaternion in [x, y, z, w] format
+
+        Returns
+        -------
+        q : numpy.ndarray
+            [x, y, z, w] quaternion
+
+        Examples
+        --------
+        >>> from numpy import pi
+        >>> from skrobot.coordinates import make_coords
+        >>> c = make_coords()
+        >>> c.quaternion_xyzw
+        array([0., 0., 0., 1.])
+        >>> c.rotate(pi / 3, 'y').rotate(pi / 5, 'z')
+        >>> c.quaternion_xyzw
+        array([0.1545085 , 0.47552826, 0.26761657, 0.8236391 ])
+        """
+        return wxyz2xyzw(matrix2quaternion(self._rotation))
+
     @property
     def dual_quaternion(self):
         """Property of DualQuaternion
@@ -765,7 +811,7 @@ class Coordinates(object):
 
     def difference_position(self, coords,
                             translation_axis=True):
-        """Return differences in positoin of given coords.
+        """Return differences in position of given coords.
 
         Parameters
         ----------
