@@ -6,6 +6,7 @@ from numbers import Number
 import sys
 
 import actionlib
+import actionlib_msgs.msg
 import control_msgs.msg
 import numpy as np
 import rospy
@@ -37,9 +38,9 @@ def _flatten(xlst):
 
 class ROSRobotInterfaceBase(object):
 
-    """ROSRobotInterface is class for interacting real robot.
+    """ROSRobotInterface is a class for interacting with a real robot.
 
-    RobotInterface is class for interacting real robot thorugh
+    ROSRobotInterface is a class for interacting with a real robot through
     JointTrajectoryAction servers and JointState topics.
 
     """
@@ -102,6 +103,8 @@ class ROSRobotInterfaceBase(object):
 
         self.robot = copy.deepcopy(robot)
         self.robot_state = dict()
+        self.moving_status = dict()
+        self.prev_moving_status = dict()
         self.controller_timeout = controller_timeout
         self.joint_action_enable = True
         self.namespace = namespace
@@ -147,9 +150,19 @@ class ROSRobotInterfaceBase(object):
         if isinstance(time, Number):
             # Normal Number disgnated Mode
             if time < fastest_time:
+                rospy.logwarn(
+                    'Time has been changed from {} to {} '
+                    'due to joint velocity limit. '
+                    'Make sure that joint limit is correctly set in urdf'.
+                    format(time, fastest_time))
                 time = fastest_time
         elif time is None:
             time = time_scale * fastest_time
+            rospy.logwarn(
+                'Time of send angle vector is set to {}. '
+                'If the speed seems slow, check if '
+                'joint velocity limit is correctly set in urdf'.
+                format(time))
         else:
             raise ValueError(
                 'time is invalid type. {}'.format(time))
@@ -170,6 +183,28 @@ class ROSRobotInterfaceBase(object):
     def set_robot_state(self, key, msg):
         self.robot_state[key] = msg
 
+    def set_moving_status(self, key, msg):
+        """Update the moving status based on follow_joint_trajectory.
+
+        While the robot is moving, the latest goal status is ACTIVE (1).
+        When the goal is achieved, its status changes to SUCCEEDED (3).
+        """
+        moving = any(
+            goal_status.status in (actionlib_msgs.msg.GoalStatus.ACTIVE,
+                                   actionlib_msgs.msg.GoalStatus.PREEMPTING)
+            for goal_status in msg.status_list
+        )
+        if moving:
+            self.moving_status[key] = True
+            self.prev_moving_status[key] = True
+        elif self.prev_moving_status.get(key, False):
+            # If all goals are completed, the status stays True
+            # for one more cycle in case a new goal is added immediately.
+            self.moving_status[key] = True
+            self.prev_moving_status[key] = False
+        else:
+            self.moving_status[key] = False
+
     def update_robot_state(self, wait_until_update=False):
         """Update robot state.
 
@@ -281,7 +316,9 @@ class ROSRobotInterfaceBase(object):
                 if self.controller_timeout is None:
                     rospy.logwarn(
                         'Waiting for actionlib interface forever '
-                        'because controler-timeout is None')
+                        'because controller-timeout is None')
+                rospy.loginfo('Waiting for actionlib interface {}'
+                              .format(action_name))
                 if not (
                     self.joint_action_enable and action.wait_for_server(
                         rospy.Duration(
@@ -289,24 +326,38 @@ class ROSRobotInterfaceBase(object):
                     rospy.logwarn('{} is not respond, {}_interface is disable'.
                                   format(action, self.robot.name))
                     rospy.logwarn('make sure that you can run '
-                                  "'rostopic echo /{0}/status' "
-                                  "and 'rostopic info /{0}/status'".
+                                  "'rostopic echo {0}/status' "
+                                  "and 'rostopic info {0}/status'".
                                   format(action_name))
                     if joint_enable_check:
                         self.joint_action_enable = False
                         return []
+                else:
+                    rospy.loginfo('Actionlib interface {} is enabled'
+                                  .format(action_name))
             for param in self.default_controller():
                 controller_state = param['controller_state']
+                trajectory_status = '{}/status'.format(
+                    param['controller_action'])
                 if self.namespace is not None:
-                    topic_name = '{}/{}'.format(
+                    controller_state_topic_name = '{}/{}'.format(
                         self.namespace,
                         controller_state)
+                    trajectory_status_topic_name = '{}/{}'.format(
+                        self.namespace,
+                        trajectory_status)
                 else:
-                    topic_name = controller_state
+                    controller_state_topic_name = controller_state
+                    trajectory_status_topic_name = trajectory_status
                 rospy.Subscriber(
-                    topic_name,
+                    controller_state_topic_name,
                     control_msgs.msg.JointTrajectoryControllerState,
                     lambda msg: self.set_robot_state(controller_state, msg))
+                rospy.Subscriber(
+                    trajectory_status_topic_name,
+                    actionlib_msgs.msg.GoalStatusArray,
+                    lambda msg: self.set_moving_status(
+                        param['controller_type'], msg))
         else:  # not creating actions, just search
             self.controller_type = controller_type
         self.controller_table[controller_type] = tmp_actions
@@ -363,7 +414,7 @@ class ROSRobotInterfaceBase(object):
                      velocities=None):
         """Send joint angle to robot
 
-        Send joint angle to robot. this method retuns immediately, so use
+        Send joint angle to robot. this method returns immediately, so use
         self.wait_interpolation to block until the motion stops.
 
         Parameters
@@ -425,7 +476,7 @@ class ROSRobotInterfaceBase(object):
         return av
 
     def potentio_vector(self):
-        """Retuns current robot angle vector, This method uses caced data."""
+        """Returns current robot angle vector, This method uses caced data."""
         return self.robot.angle_vector()
 
     def send_ros_controller(
@@ -488,7 +539,7 @@ class ROSRobotInterfaceBase(object):
                               time_scale=5.0):
         """Send sequence of joint angles to robot
 
-        Send sequence of joint angle to robot, this method retuns
+        Send sequence of joint angle to robot, this method returns
         immediately, so use self.wait_interpolation to block until the motion
         stops.
 
@@ -584,8 +635,8 @@ class ROSRobotInterfaceBase(object):
 
         Returns
         -------
+        list[bool]
             return values are a list of is_interpolating for all controllers.
-            if all interpolation has stopped, return True.
         """
         if controller_type:
             controller_actions = self.controller_table[controller_type]
@@ -594,8 +645,29 @@ class ROSRobotInterfaceBase(object):
         for action in controller_actions:
             # TODO(simultaneously wait_for_result)
             action.wait_for_result(timeout=rospy.Duration(timeout))
-        # TODO(Fix return value)
-        return True
+        is_interpolatings = map(
+            lambda action: action.is_interpolating(), controller_actions)
+        return list(is_interpolatings)
+
+    def is_interpolating(self, controller_type=None):
+        if controller_type:
+            controller_actions = self.controller_table[controller_type]
+        else:
+            controller_actions = self.controller_table[self.controller_type]
+        is_interpolatings = map(
+            lambda action: action.is_interpolating(), controller_actions)
+        return any(list(is_interpolatings))
+
+    def is_moving(self, controller_type=None):
+        """"Check whether the robot is moving due to follow_joint_trajectory.
+
+        This is not limited to goals sent from the same instance.
+        """
+        if controller_type is None or controller_type == self.controller_type:
+            is_movings = list(self.moving_status.values())
+        else:
+            is_movings = [self.moving_status[controller_type]]
+        return any(is_movings)
 
     def angle_vector_duration(self, start_av, end_av, controller_type=None):
         """Calculate maximum time to reach goal for all joint.
@@ -636,6 +708,13 @@ class ROSRobotInterfaceBase(object):
             time_list.append(time)
         return max(time_list)
 
+    def cancel_angle_vector(self):
+        """Stop motion via follow joint trajectory cancel
+
+        """
+        for action in self.controller_actions:
+            action.cancel_all_goals()
+
 
 class ControllerActionClient(actionlib.SimpleActionClient):
 
@@ -650,4 +729,4 @@ class ControllerActionClient(actionlib.SimpleActionClient):
         self.last_feedback_msg_stamp = msg.header.stamp
 
     def is_interpolating(self):
-        return self.time_sequence is None
+        return not self.simple_state == actionlib.SimpleGoalState.DONE
