@@ -153,6 +153,10 @@ class Joint(object):
         self.default_coords = self.child_link.copy_coords()
         self._hooks = hooks
 
+    @property
+    def type(self):
+        raise NotImplementedError('Joint type must be defined')
+
     @property
     def joint_dof(self):
         raise NotImplementedError
@@ -210,6 +214,12 @@ class RotationalJoint(Joint):
             logger.warning(message)
             self.max_joint_velocity = np.deg2rad(5)
 
+    @property
+    def type(self):
+        if np.isinf(self.min_angle) or np.isinf(self.max_angle):
+            return 'continuous'
+        return 'revolute'
+
     def joint_angle(self, v=None, relative=None, enable_hook=True):
         """Return joint angle.
 
@@ -285,6 +295,10 @@ class FixedJoint(Joint):
         self.joint_acceleration = 0.0  # [rad/s^2]
         self.joint_torque = 0.0  # [Nm]
 
+    @property
+    def type(self):
+        return 'fixed'
+
     def joint_angle(self, v=None, relative=None, enable_hook=True):
         """Joint angle method.
 
@@ -373,6 +387,10 @@ class LinearJoint(Joint):
             logger.warning(message)
             self.max_joint_velocity = np.pi / 4.0
 
+    @property
+    def type(self):
+        return 'prismatic'
+
     def joint_angle(self, v=None, relative=None, enable_hook=True):
         """Return this joint's linear translation (joint angle).
 
