@@ -10,8 +10,8 @@ import numpy as np
 import numpy.linalg as LA
 from ordered_set import OrderedSet
 import six
-import trimesh
 
+from skrobot._lazy_imports import _lazy_trimesh
 from skrobot.coordinates import CascadedCoords
 from skrobot.coordinates import convert_to_axis_vector
 from skrobot.coordinates import Coordinates
@@ -88,9 +88,9 @@ class CascadedLink(CascadedCoords):
     def _is_relevant(self, joint, something):
         """check if `joint` affects `something`
 
-        `somthing` must be at least CascadedCoords and must be
+        `something` must be at least CascadedCoords and must be
         connected to this CascadedLink. Otherwirse, this method
-        raise AssertionError. If `something` is a decendant of `joint`,
+        raise AssertionError. If `something` is a descendant of `joint`,
         which means movement of `joint` affects `something`, thus
         this method returns `True`. Otherwise returns `False`.
         """
@@ -301,7 +301,7 @@ class CascadedLink(CascadedCoords):
         # a : avoid-collision-joint-gain
         # b : avoid-collision-null-gain
         #
-        # implimentation issue:
+        # implementation issue:
         # when link and object are collide,
         # p = (nearestpoint_on_object_surface - center_of_link )
         # else
@@ -835,7 +835,7 @@ class CascadedLink(CascadedCoords):
             rotation_axis = []
         c = self.calc_target_joint_dimension(
             union_link_list)
-        # add dimensions of additonal-jacobi
+        # add dimensions of additional-jacobi
         r = self.calc_target_axis_dimension(
             rotation_axis, translation_axis) + additional_jacobi_dimension
 
@@ -898,6 +898,9 @@ class CascadedLink(CascadedCoords):
         # store current angle vector
         joint_list = list(
             set([l.joint for l in union_link_list] + self.joint_list))
+        if None in joint_list:
+            logger.error('All links in link_list must have a parent joint')
+            return True
         av0 = [j.joint_angle() for j in joint_list]
         c0 = None
         if self.parent is None:
@@ -1035,7 +1038,7 @@ class CascadedLink(CascadedCoords):
         self.reset_joint_angle_limit_weight(union_link_list)
 
         # TODO(add collision check)
-        if success:
+        if success or not revert_if_fail:
             return self.angle_vector()
 
         # reset angle vector
@@ -1620,6 +1623,11 @@ class CascadedLink(CascadedCoords):
     def self_collision_check(self):
         """Return collision link pair
 
+        Note: This function uses trimesh.collision.CollisionManager internally.
+        We need to install python-fcl to use this function.
+        If you want to use this function, please install python-fcl by
+        `pip install python-fcl`.
+
         Returns
         -------
         is_collision : bool
@@ -1631,6 +1639,7 @@ class CascadedLink(CascadedCoords):
             that the two corresponding objects are in collision.
         """
         if self._collision_manager is None:
+            trimesh = _lazy_trimesh()
             self._collision_manager = trimesh.collision.CollisionManager()
             for link in self.link_list:
                 transform = link.worldcoords().T()
@@ -1678,7 +1687,11 @@ class RobotModel(CascadedLink):
         raise NotImplementedError()
 
     def init_pose(self):
-        return self.angle_vector(np.zeros_like(self.angle_vector()))
+        target_angles = np.zeros_like(self.joint_min_angles)
+        target_angles = np.clip(target_angles,
+                                self.joint_min_angles,
+                                self.joint_max_angles)
+        return self.angle_vector(target_angles)
 
     def _meshes_from_urdf_visuals(self, visuals):
         meshes = []
@@ -1691,8 +1704,11 @@ class RobotModel(CascadedLink):
             raise TypeError('visual must be urdf.Visual, but got: {}'
                             .format(type(visual)))
 
+        trimesh = None
         meshes = []
         for mesh in visual.geometry.meshes:
+            if trimesh is None:
+                trimesh = _lazy_trimesh()
             mesh = mesh.copy()
 
             # rescale
