@@ -6,10 +6,11 @@ import os
 import shutil
 import tempfile
 
+import filelock
 import numpy as np
 import pysdfgen
-from scipy.interpolate import RegularGridInterpolator
 
+from skrobot._lazy_imports import _lazy_scipy
 from skrobot.coordinates import CascadedCoords
 from skrobot.coordinates import Coordinates
 from skrobot.data import get_cache_dir
@@ -47,7 +48,7 @@ def trimesh2sdf(mesh, **gridsdf_kwargs):
             radius = mesh.metadata['radius']
             sdf = SphereSDF(radius)
         else:
-            msg = "primtive type {0} is not supported".format(shape)
+            msg = "primitive type {0} is not supported".format(shape)
             raise ValueError(msg)
 
         if "original_primitive_origin_for_sdf" in mesh.metadata:
@@ -249,7 +250,7 @@ class UnionSDF(SignedDistanceFunction):
         return sd_vals_union
 
     def _surface_points(self, n_sample=1000):
-        # equaly asign sample number to each sdf.surface_points()
+        # equally assign sample number to each sdf.surface_points()
         n_list = len(self.sdf_list)
         n_sample_each = int(floor(n_sample / n_list))
         n_sample_last = n_sample - n_sample_each * (n_list - 1)
@@ -398,7 +399,14 @@ class GridSDF(SignedDistanceFunction):
         # create regular grid interpolator
         xlin, ylin, zlin = [
             np.array(range(d)) * resolution for d in self._data.shape]
-        self.itp = RegularGridInterpolator(
+        scipy = _lazy_scipy()
+        try:
+            interpolator = scipy.interpolate.RegularGridInterpolator
+        except AttributeError:
+            # scipy<=1.8.0
+            from scipy.interpolate import \
+                RegularGridInterpolator as interpolator
+        self.itp = interpolator(
             (xlin, ylin, zlin),
             self._data,
             bounds_error=False,
@@ -420,7 +428,7 @@ class GridSDF(SignedDistanceFunction):
         -------
         is_out_arr : numpy.ndarray[bool](n_points,)
             If points is out of the interpolator's boundary,
-            the correspoinding element of is_out_arr is True
+            the corresponding element of is_out_arr is True
         """
         points_sdf \
             = super(GridSDF, self)._transform_pts_world_to_sdf(points_world)
@@ -520,13 +528,18 @@ class GridSDF(SignedDistanceFunction):
             checksum_md5(obj_filepath), dim_grid, padding_grid)
 
         sdf_cache_path = os.path.join(sdf_cache_dir, hashed_filename + '.sdf')
-        if not os.path.exists(sdf_cache_path):
-            logger.info(
-                'pre-computing sdf and making a cache at {0}.'
-                .format(sdf_cache_path))
-            pysdfgen.obj2sdf(str(obj_filepath), dim_grid, padding_grid,
-                             output_filepath=sdf_cache_path)
-            logger.info('finish pre-computation')
+        lock = filelock.FileLock(sdf_cache_path + '.lock')
+        with lock:
+            if not os.path.exists(sdf_cache_path):
+                logger.info(
+                    'trying to acquire lock for {0}...'
+                    .format(sdf_cache_path))
+                logger.info(
+                    'pre-computing sdf and making a cache at {0}.'
+                    .format(sdf_cache_path))
+                pysdfgen.obj2sdf(str(obj_filepath), dim_grid, padding_grid,
+                                 output_filepath=sdf_cache_path)
+                logger.info('finish pre-computation')
         return GridSDF.from_file(sdf_cache_path, **kwargs)
 
 
