@@ -22,17 +22,14 @@ except ImportError:
 from lxml import etree as ET
 import networkx as nx
 import numpy as np
-import PIL
+import PIL.Image
 import six
-import trimesh
-from trimesh.visual.material import PBRMaterial
 
+from skrobot._lazy_imports import _lazy_trimesh
 from skrobot.coordinates import normalize_vector
 from skrobot.coordinates import rpy_angle
 from skrobot.coordinates import rpy_matrix
 from skrobot.pycompat import lru_cache
-from skrobot.utils.mesh import auto_simplify_quadric_decimation
-from skrobot.utils.mesh import split_mesh_by_face_color
 
 
 try:
@@ -41,6 +38,7 @@ except ImportError:
     rospkg = None
 
 
+mesh_filenames_cache = {}
 logger = getLogger(__name__)
 _CONFIGURABLE_VALUES = {"mesh_simplify_factor": np.inf,
                         'no_mesh_load_mode': False,
@@ -48,6 +46,8 @@ _CONFIGURABLE_VALUES = {"mesh_simplify_factor": np.inf,
                         'decimation_area_ratio_threshold': None,
                         'simplify_vertex_clustering_voxel_size': None,
                         'enable_mesh_cache': False,
+                        'force_visual_mesh_origin_to_zero': False,
+                        'overwrite_mesh': False,
                         }
 _MESH_CACHE = {}
 
@@ -70,16 +70,19 @@ def no_mesh_load_mode():
 def export_mesh_format(
         mesh_format,
         decimation_area_ratio_threshold=None,
-        simplify_vertex_clustering_voxel_size=None):
+        simplify_vertex_clustering_voxel_size=None,
+        overwrite_mesh=False):
     _CONFIGURABLE_VALUES["export_mesh_format"] = mesh_format
     _CONFIGURABLE_VALUES["decimation_area_ratio_threshold"] = \
         decimation_area_ratio_threshold
     _CONFIGURABLE_VALUES["simplify_vertex_clustering_voxel_size"] = \
         simplify_vertex_clustering_voxel_size
+    _CONFIGURABLE_VALUES["overwrite_mesh"] = overwrite_mesh
     yield
     _CONFIGURABLE_VALUES["export_mesh_format"] = None
     _CONFIGURABLE_VALUES["decimation_area_ratio_threshold"] = None
     _CONFIGURABLE_VALUES["simplify_vertex_clustering_voxel_size"] = None
+    _CONFIGURABLE_VALUES["overwrite_mesh"] = False
 
 
 @contextlib.contextmanager
@@ -89,6 +92,13 @@ def enable_mesh_cache():
     _CONFIGURABLE_VALUES['enable_mesh_cache'] = False
 
 
+@contextlib.contextmanager
+def force_visual_mesh_origin_to_zero():
+    _CONFIGURABLE_VALUES['force_visual_mesh_origin_to_zero'] = True
+    yield
+    _CONFIGURABLE_VALUES['force_visual_mesh_origin_to_zero'] = False
+
+
 def get_transparency(mesh):
     if hasattr(mesh, 'visual') and hasattr(mesh.visual, 'material'):
         material = mesh.visual.material
@@ -284,6 +294,7 @@ def _load_meshes(filename):
     meshes : list of :class:`~trimesh.base.Trimesh`
         The meshes loaded from the file.
     """
+    trimesh = _lazy_trimesh()
     try:
         _, ext = os.path.splitext(filename)
         # It seems that .3DXML files assume [mm] unit.
@@ -322,7 +333,8 @@ def _load_meshes(filename):
     for mesh in meshes:
         transparency = get_transparency(mesh)
         if transparency is not None and transparency == 0.0:
-            if isinstance(mesh.visual.material, PBRMaterial):
+            if isinstance(mesh.visual.material,
+                          trimesh.visual.material.PBRMaterial):
                 mesh.visual.material.baseColorFactor[3] = 255
     return meshes
 
@@ -506,7 +518,8 @@ class URDFType(object):
             and elements in the class arrays.
         """
         kwargs = cls._parse_simple_attribs(node)
-        kwargs.update(cls._parse_simple_elements(node, path))
+        if node is not None:
+            kwargs.update(cls._parse_simple_elements(node, path))
         return kwargs
 
     @classmethod
@@ -678,6 +691,7 @@ class Box(URDFType):
         """
         if len(self._meshes) == 0:
             if _CONFIGURABLE_VALUES['no_mesh_load_mode'] is False:
+                trimesh = _lazy_trimesh()
                 self._meshes = [trimesh.creation.box(extents=self.size)]
         return self._meshes
 
@@ -738,6 +752,7 @@ class Cylinder(URDFType):
         """
         if len(self._meshes) == 0:
             if _CONFIGURABLE_VALUES['no_mesh_load_mode'] is False:
+                trimesh = _lazy_trimesh()
                 self._meshes = [trimesh.creation.cylinder(
                     radius=self.radius, height=self.length
                 )]
@@ -782,6 +797,7 @@ class Sphere(URDFType):
         """
         if len(self._meshes) == 0:
             if _CONFIGURABLE_VALUES['no_mesh_load_mode'] is False:
+                trimesh = _lazy_trimesh()
                 self._meshes = [trimesh.creation.icosphere(radius=self.radius)]
         return self._meshes
 
@@ -853,17 +869,27 @@ class Mesh(URDFType):
     def meshes(self, value):
         if isinstance(value, six.string_types):
             value = load_meshes(value)
-        elif isinstance(value, (list, tuple, set)):
+            self._meshes = value
+            return
+
+        if isinstance(value, (list, tuple, set)):
             value = list(value)
-            for m in value:
-                if not isinstance(m, trimesh.Trimesh):
-                    raise TypeError('Mesh requires a trimesh.Trimesh or a '
-                                    'list of them')
-        elif isinstance(value, trimesh.Trimesh):
-            value = [value]
-        else:
-            raise TypeError('Mesh requires a trimesh.Trimesh')
-        self._meshes = value
+            trimesh = None
+            for i, m in enumerate(value):
+                if not hasattr(m, '__module__') or m.__module__ != 'trimesh':
+                    if trimesh is None:
+                        trimesh = _lazy_trimesh()
+                    if not isinstance(m, trimesh.Trimesh):
+                        raise TypeError('Mesh requires a trimesh.Trimesh or a '
+                                        'list of them')
+            self._meshes = value
+            return
+
+        trimesh = _lazy_trimesh()
+        if isinstance(value, trimesh.Trimesh):
+            self._meshes = [value]
+            return
+        raise TypeError('Mesh requires a trimesh.Trimesh')
 
     @classmethod
     def _from_xml(cls, node, path):
@@ -871,8 +897,8 @@ class Mesh(URDFType):
 
         # Load the mesh, combining collision geometry meshes but keeping
         # visual ones separate to preserve colors and textures
-        fn = get_filename(path, kwargs['filename'])
         if _CONFIGURABLE_VALUES['no_mesh_load_mode'] is False:
+            fn = get_filename(path, kwargs['filename'])
             meshes = load_meshes(fn)
         else:
             meshes = []
@@ -898,6 +924,7 @@ class Mesh(URDFType):
         meshes = self.meshes
         if _CONFIGURABLE_VALUES[
                 "decimation_area_ratio_threshold"] is not None:
+            from skrobot.utils.mesh import auto_simplify_quadric_decimation
             meshes = auto_simplify_quadric_decimation(
                 meshes, area_ratio_threshold=_CONFIGURABLE_VALUES[
                     "decimation_area_ratio_threshold"])
@@ -908,23 +935,30 @@ class Mesh(URDFType):
                 _CONFIGURABLE_VALUES['simplify_vertex_clustering_voxel_size'],
             )
 
+        trimesh = _lazy_trimesh()
         if fn.endswith('.dae'):
+            from skrobot.utils.mesh import split_mesh_by_face_color
             export_meshes = []
             has_texture_visual = False
             for mesh in meshes:
                 has_texture_visual |= mesh.visual.kind == 'texture'
                 export_meshes.extend(split_mesh_by_face_color(mesh))
             meshes = export_meshes
-            if not (os.path.exists(fn) and has_texture_visual):
+            if _CONFIGURABLE_VALUES['overwrite_mesh'] is True \
+                    or not (os.path.exists(fn) and has_texture_visual):
                 # don't overwrite textured mesh.
                 dae_data = trimesh.exchange.dae.export_collada(meshes)
                 with open(fn, 'wb') as f:
                     f.write(dae_data)
         elif fn.endswith('.stl'):
-            meshes = trimesh.util.concatenate(meshes)
-            trimesh.exchange.export.export_mesh(meshes, fn)
+            if _CONFIGURABLE_VALUES['overwrite_mesh'] \
+                    or not os.path.exists(fn):
+                meshes = trimesh.util.concatenate(meshes)
+                trimesh.exchange.export.export_mesh(meshes, fn)
         else:
-            trimesh.exchange.export.export_mesh(meshes, fn)
+            if _CONFIGURABLE_VALUES['overwrite_mesh'] \
+                    or not os.path.exists(fn):
+                trimesh.exchange.export.export_mesh(meshes, fn)
 
         # Unparse the node
         node = self._unparse(path)
@@ -958,9 +992,9 @@ class Geometry(URDFType):
     _TAG = 'geometry'
 
     def __init__(self, box=None, cylinder=None, sphere=None, mesh=None):
-        if (box is None and cylinder is None
-                and sphere is None and mesh is None):
-            raise ValueError('At least one geometry element must be set')
+        # if (box is None and cylinder is None
+        #         and sphere is None and mesh is None):
+        #     raise ValueError('At least one geometry element must be set')
         self.box = box
         self.cylinder = cylinder
         self.sphere = sphere
@@ -1042,7 +1076,10 @@ class Geometry(URDFType):
         list of :class:`~trimesh.base.Trimesh` : The geometry's triangular
         mesh representation(s).
         """
-        return self.geometry.meshes
+        geo = self.geometry
+        if geo is None:
+            return []
+        return geo.meshes
 
 
 class Texture(URDFType):
@@ -1242,6 +1279,18 @@ class Collision(URDFType):
         self.geometry = geometry
         self.name = name
         self.origin = origin
+        if _CONFIGURABLE_VALUES['force_visual_mesh_origin_to_zero']:
+            if self.geometry.mesh.filename not in mesh_filenames_cache:
+                mesh_filenames_cache[self.geometry.mesh.filename] = True
+                if self.geometry.mesh is not None:
+                    new_mesh = []
+                    for mesh in self.geometry.meshes:
+                        mesh.apply_transform(self.origin)
+                        new_mesh.append(mesh)
+                    self.geometry.mesh.meshes = new_mesh
+                    del new_mesh
+            # force the origin to be zero
+            self.origin = np.eye(4)
 
     @property
     def geometry(self):
@@ -1322,6 +1371,19 @@ class Visual(URDFType):
         self.origin = origin
         self.material = material
 
+        if _CONFIGURABLE_VALUES['force_visual_mesh_origin_to_zero']:
+            if self.geometry.mesh.filename not in mesh_filenames_cache:
+                mesh_filenames_cache[self.geometry.mesh.filename] = True
+                if self.geometry.mesh is not None:
+                    new_mesh = []
+                    for mesh in self.geometry.meshes:
+                        mesh.apply_transform(self.origin)
+                        new_mesh.append(mesh)
+                    self.geometry.mesh.meshes = new_mesh
+                    del new_mesh
+            # force the origin to be zero
+            self.origin = np.eye(4)
+
     @property
     def geometry(self):
         """:class:`.Geometry` : The geometry of this element.
@@ -2383,6 +2445,7 @@ class Joint(URDFType):
                 cfg = 0.0
             else:
                 cfg = float(cfg)
+            trimesh = _lazy_trimesh()
             R = trimesh.transformations.rotation_matrix(cfg, self.axis)
             return self.origin.dot(R)
         elif self.joint_type == 'prismatic':
@@ -2457,7 +2520,7 @@ class Link(URDFType):
         The inertial properties of the link.
     visuals : list of :class:`.Visual`, optional
         The visual properties of the link.
-    collsions : list of :class:`.Collision`, optional
+    collisions : list of :class:`.Collision`, optional
         The collision properties of the link.
     """
 
@@ -2501,7 +2564,7 @@ class Link(URDFType):
     @inertial.setter
     def inertial(self, value):
         if value is not None and not isinstance(value, Inertial):
-            raise TypeError('Expected Intertial object')
+            raise TypeError('Expected Inertial object')
         self._inertial = value
 
     @property
